// PICSLabelList.java
// by Yang-hua Chu
// last update: 10/11/96

package w3c.pics.referee;

import java.io.* ;
import java.net.*;
import java.util.*;

public class PICSLabelList 
{
  String version = null;
  boolean isNoRatings = false;
  String noRatingError = "";

  Vector labels = new Vector();  // a vector of PICSLabel


  // throws PICSParseException if the content of ins stream is not
  // a legal labelist.
  // the stream forwards to the last token read.


  public void parse(SExp exp)
       throws PICSParseException
  {
    // labellist :: '(' [version] service-info+ ')'
    if (!exp.expect(exp.SEXP_IDENT))
      throw new PICSParseException(exp.errorMsg("label-version"));
    this.version = exp.current().getIdent();
    
    // service-info :: ['error'] '(no-ratings' explanation* ')'
    if (exp.expect(exp.SEXP_IDENT, "error")) {
      isNoRatings = true;

      if (!exp.expect(exp.SEXP_SEXP))
	throw new PICSParseException(exp.errorMsg("(no-ratings...)"));
      
      SExp exp2 = exp.current();
      if (! exp2.expect(exp2.SEXP_IDENT, "no-ratings")) 
	throw new PICSParseException(exp2.errorMsg("no-ratings"));
      
      // service-info :: 'error' '(no-ratings' [explanation*] ')'
      while (exp2.expect(exp2.SEXP_STRING))   
	noRatingError = noRatingError + " " + exp2.current().getString();
      
      if (! exp2.isLast())	
	throw new PICSParseException(exp2.errorMsg("explanation error"));

      return;
    }
    
    // service-info :: serviceID ...
    do {
      PICSLabel label = new PICSLabel(version);
      label.parse(exp);
      labels.addElement(label);
    } while (! exp.isLast());
  }
  
  

  public String toString() 
  {
    String s="(" + this.version + " ";
    if (isNoRatings)
      if (noRatingError.equals(""))
	return (s + "error (no-ratings ) )");
      else 
	return (s + "error (no-ratings \"" + noRatingError + "\" ) )");
    
    for (int i=0; i<labels.size(); i++)
      s += ((PICSLabel) labels.elementAt(i)).toString() + "\n";
    return (s +=  ")");
  }
  
  public static void main(String[] argv) 
       throws IOException, PICSParseException, SExpException
  { 

    SExp exp = new SExp();
    exp.parse(new FileInputStream(argv[0]));
    System.out.println(exp.toString());

    PICSLabelList ll = new PICSLabelList();
    ll.parse(exp);
    System.out.println(ll.toString());
    
    /*    do {
      PICSLabelList ll = null;
      try {
	ll = new PICSLabelList(exp);
	System.out.println(ll.toString());
      }
      catch (PICSParseException s) {
	System.out.println(s.getMessage());
	exp.nextToken();
      }
      System.out.println("------------------------------------");
    } while (!exp.expect(exp.SEXP_EOF));
    */
  } //main

}



