// PICSLabelList.java
// by Yang-hua Chu
// last update: 10/11/96

package w3c.pics.label;

import w3c.pics.referee.*;
import java.io.* ;
import java.net.*;
import java.util.*;
import w3c.pics.util.*;

public class PICSLabelList 
{
  private boolean hasRatings = false;
  private String version = "";
  private String noRatingError = "";

  private PICSLabel labels[] = {};  // a vector of PICSLabel

  // throws PICSParseException if the content of ins stream is not
  // a legal labelist.
  // the stream forwards to the last token read.

  public boolean hasRatings()
  {
    return hasRatings;
  }
  
  public String getVersion()
  {
    return version;
  }

  // index starts from 0
  public PICSLabel getPICSLabel(int index)
    throws PICSLabelException
  {
    try { return labels[index]; }
    catch (Exception e) {
      throw new PICSLabelException("Index out of bound: " + index);
    }
  }
  
  // the number of labels (service-info) in the labellist
  public int size()
  {
    return labels.length;
  }
  

  // add a PICS label to the label-list
  public int addPICSLabel(PICSLabel inLabel)
  {
    PICSLabel nlabels[] = new PICSLabel[size() + 1] ;
    System.arraycopy (labels, 0, nlabels, 0, size()) ;
    labels = nlabels;
    labels[size()-1] = inLabel;
    return (size()-1);
  }
  

  public void parse(SExp exp)
       throws PICSParseException
  {
    // labellist :: '(' [version] service-info+ ')'
    if (!exp.expect(exp.SEXP_IDENT))
      throw new PICSParseException(exp.errorMsg("label-version"));
    this.version = exp.current().getIdent();
    
    // service-info :: ['error'] '(no-ratings' explanation* ')'
    if (exp.expect(exp.SEXP_IDENT, "error")) {
      hasRatings = false;

      if (!exp.expect(exp.SEXP_SEXP))
	throw new PICSParseException(exp.errorMsg("(no-ratings...)"));
      
      SExp exp2 = exp.current();
      if (! exp2.expect(exp2.SEXP_IDENT, "no-ratings")) 
	throw new PICSParseException(exp2.errorMsg("no-ratings"));
      
      // service-info :: 'error' '(no-ratings' [explanation*] ')'
      while (exp2.expect(exp2.SEXP_STRING))   
	noRatingError = noRatingError + " " + exp2.current().getString();
      
      if (! exp2.isLast())	
	throw new PICSParseException(exp2.errorMsg("explanation error"));

      return;
    }
    
    hasRatings = true;
    // service-info :: serviceID ...
    do {
      PICSLabel label = new PICSLabel(version);
      label.parse(exp);
      addPICSLabel(label);
    } while (! exp.isLast());
  }
  

  public String toString() 
  {
    String s = "( " + version + " ";
    if (! hasRatings()) 
      if (noRatingError.equals("")) 
	return (s + "error (no-ratings ) )");
      else 
	return (s + "error (no-ratings \"" + noRatingError + "\" ) )");
    
    for (int i=0; i<size(); i++) {
      try {s += getPICSLabel(i).toString() + "\n";}
      catch (PICSLabelException e) { break; }
    }
    return (s +=  ")");
  }
  


  public static void main(String[] argv) 
       throws Exception
  { 

    SExp exp = new SExp();
    exp.parse(new FileInputStream(argv[0]));
    System.out.println(exp.toString());

    PICSLabelList ll = new PICSLabelList();
    ll.parse(exp);
    System.out.println(ll.toString());

    /*    do {
      PICSLabelList ll = null;
      try {
	ll = new PICSLabelList(exp);
	System.out.println(ll.toString());
      }
      catch (PICSParseException s) {
	System.out.println(s.getMessage());
	exp.nextToken();
      }
      System.out.println("------------------------------------");
    } while (!exp.expect(exp.SEXP_EOF));
    */
  } //main

}



