// Profiles.java
// by Yang-hua Chu
// last update: 10/11/96

package w3c.pics.referee;

import java.io.* ;
import java.net.*;
import java.util.*;
import w3c.pics.util.*;
import w3c.pics.label.*;

public class LabelParser
{
  private PICSLabelList list[] = {};


  public int addLabel(PICSLabelList inList)
  {
    PICSLabelList nlist[] = new PICSLabelList[size() + 1] ;
    System.arraycopy (list, 0, nlist, 0, size()) ;
    list = nlist;
    list[size()-1] = inList;
    return (size()-1);
  }


  public PICSLabelList getLabel(int index)
    throws ProfilesException
  {
    try { return list[index]; }
    catch (Exception e) {
      throw new ProfilesException("Index out of bound: " + index);
    }
  }


  public int size()
  {
    return list.length;
  }


  private SExp makePair(String key, String value)
  {
    SExp exp = new SExp(SExp.SEXP_SEXP);
    exp.addIdent(key);
    exp.addString(value);
    return exp;
  }

  private SExp makePair(String key, SExp value)
  {
    SExp exp = new SExp(SExp.SEXP_SEXP);
    exp.addIdent(key);
    exp.addSExp(value);
    return exp;
  }

  public SExp search(URL targetURL, String targetService,
		     boolean allowGeneric)
    throws PICSLabelException, ProfilesException
  {
    SExp result = new SExp(SExp.SEXP_SEXP);
    
    // iterate through all labels in 'list'
    for (int i=0; i<size(); i++) {
      PICSLabelList cur_label_list = getLabel(i);
      SExp version_sexp = makePair("version", cur_label_list.getVersion());
      SExp original_sexp = makePair("original", 
				    new SExp(cur_label_list.toString()));
      
      for (int j=0; j<cur_label_list.size(); j++) {
	PICSLabel cur_label = cur_label_list.getPICSLabel(j); 
	SExp service_sexp =makePair("service", cur_label.serviceID.toString());
	for (int k=0; k<cur_label.size(); k++) {
	  PICSSingleLabel cur_single_label = cur_label.getSingleLabel(k);
	  if (cur_single_label.options.isFor(targetURL, allowGeneric)) {
	    SExp one = new SExp(SExp.SEXP_SEXP);
	    one.addSExp(version_sexp);
	    one.addSExp(service_sexp);
	    one.addSExp(original_sexp);
	    SExp opts = parseOptions(cur_single_label.options.merge(cur_label.options));
	    for (int m=1; m<=opts.size(); m++) 
	      one.addSExp(opts.getSExp(m));
	    one.addSExp(parseRatings(cur_single_label.ratings));
	    result.addSExp(one);
	    break;   // only interested in finding one match
	  }
	}
      }
    }
    return result;
  }
  

  public SExp parseRatings(PICSLabelRatings ratings)
    throws PICSLabelException
  {
    SExp result = new SExp(SExp.SEXP_SEXP);
    result.addIdent("ratings");
    SExp exp = new SExp("(" + ratings.toString() + ")");
    exp = exp.getSExp(2);
    for (int i=1; i<=exp.size(); i=i+2)
      result.addSExp(makePair(exp.getSExp(i).getString(), exp.getSExp(i+1)));
    return result;
  }


  public SExp parseOptions(PICSLabelOptions opts)
  {
    SExp exp = new SExp(SExp.SEXP_SEXP);
    SExp opts_sexp = new SExp("("+ opts.toString()+")");
    for (int i=1; i<=opts_sexp.size(); i=i+2) {
      SExp nopt = new SExp(SExp.SEXP_SEXP);
      nopt.addSExp(opts_sexp.getSExp(i));
      nopt.addSExp(opts_sexp.getSExp(i+1));
      exp.addSExp(nopt);
    }
    return exp;
  }
  
  
  public static void main(String[] argv) 
    throws Exception
  {
    
    SExp exp = new SExp();
    exp.parse(new FileInputStream(argv[0]));
    System.out.println(exp.toString());
    
    PICSLabelList ll = new PICSLabelList();
    ll.parse(exp);
    System.out.println(ll.toString());

    LabelParser lp = new LabelParser();
    lp.addLabel(ll);
    
    URL targetURL = new URL(argv[1]);
    System.out.println(lp.search(targetURL, "...", true).toString());
    return;
  }
}





