// Profiles.java
// by Yang-hua Chu
// last update: 10/11/96

package w3c.pics.referee;

import java.io.* ;
import java.net.*;
import java.util.*;
import w3c.pics.util.*;
import w3c.pics.label.*;

public class Profiles {
  
  public boolean DEBUG=false;
  public boolean DEBUG_TERMINAL=false;
  public URL ARG_URL = null;
  public StatementList INITIAL_STATEMENT_LIST = new StatementList();
  private ProgramDatabase PROGRAM_DATABASE = new ProgramDatabase();
  public String PROGRAM_NAME = "Profiles";
  public StatementList addedStatementList = new StatementList();
  public SExp traceSExp = new SExp(SExp.SEXP_SEXP);

  public String endorsed_rater[] = null;


  public String trace()
  {
    String s = "";
    for (int i=1; i<traceSExp.size(); i=i+2) {
      s += "eval: \n" + traceSExp.getSExp(i).toString() + "\n\n";
      s += "result: \n" + traceSExp.getSExp(i+1).toString() + "\n";
      s += "------------------------------------------------------------\n\n";
    }
    return s;
  }

  public Profiles(ProgramDatabase inputProgDatabase, 
		  StatementList statementList, 
		  URL inputURL) 
    throws ProfilesException
  {
    PROGRAM_DATABASE = inputProgDatabase;
    INITIAL_STATEMENT_LIST = statementList;
    ARG_URL = inputURL;
  }
  

  public ReturnedValues eval(String action)
    throws SExpException, ProfilesException
  {
    SExp myProgram = new SExp();
    ReturnedValues rvalue = new ReturnedValues();
    String policyString = "";
    
    try {
      policyString = PROGRAM_DATABASE.getProgram(action);
    } catch (ProfilesException e) {
      throw new ProfilesException("Unable to find the program " +action +
				  " in the program database"); 
    }
    
    if (! myProgram.parse(policyString))
	throw new ProfilesException ("invalid s-expression program file");
    
    for (int i=1; i <= myProgram.size(); i++)
      rvalue = evalRule(myProgram.getSExp(i));
    return rvalue;
  }
  

  // rule::let-rule | explicit-rule | combine-rule | invoke-rule | 
  //       install-rule | projection-rule | match-rule | variable-name 
  public ReturnedValues evalRule(SExp exp) 
    throws ProfilesException
  {
    ReturnedValues rval = null;
    
    if (exp.expect(exp.SEXP_IDENT, "url-match"))
      rval = explicitRule(exp);
    
    /*    if (exp.expect(exp.SEXP_IDENT, "let"))
	  return letRule(exp);
	  */

    if (exp.expect(exp.SEXP_IDENT, "false-if-unknown") || 
	exp.expect(exp.SEXP_IDENT, "true-if-unknown") ||
	exp.expect(exp.SEXP_IDENT, "not")) 
      rval = unaryCombine(exp);
    
    if (exp.expect(exp.SEXP_IDENT, "or"))
      rval = combineRuleOr(exp);
    
    if (exp.expect(exp.SEXP_IDENT, "and"))
      rval = combineRuleAnd(exp);
    
    if (exp.expect(exp.SEXP_IDENT, "invoke"))
      rval = invokeRule(exp);
    
    /*
      if ((exp.expect(exp.SEXP_IDENT, "install-program")) ||
	(exp.expect(exp.SEXP_IDENT, "install-interpreter")))
      return installRule(exp, exp.getString());

    if ((exp.expect(exp.SEXP_IDENT, "tri-value")) ||
	(exp.expect(exp.SEXP_IDENT, "statement-list")))
      return projectionRule(exp, exp.getString());
      */
    
    if (exp.expect(exp.SEXP_IDENT, "match"))
      rval = matchRule(exp);
    
    if (rval == null)
      throw new ProfilesException("unknown rule" + exp.toStringShort());
    
    if (DEBUG) System.out.println("Eval:\n" + exp.toString() + "\n");
    if (DEBUG_TERMINAL) halt();

    addedStatementList.merge(rval.getStatementList());
    if (DEBUG) System.out.println("Result:\n" + rval.toString() + "\n");
    if (DEBUG_TERMINAL) halt();

    traceSExp.addSExp(exp);
    traceSExp.addSExp(new SExp(rval.toString()));
    return rval;
  }


  // (false-if-unknown rule)
  // (true-if-unknown rule)
  // (not rule)
  private ReturnedValues unaryCombine(SExp exp)
    throws ProfilesException
  {
    ReturnedValues rval = evalRule(exp.getSExp(2));

    if (exp.getSExp(1).getString().equals("not")) 
      rval.setTriValue(rval.getTriValue().not());
    
    if (exp.getSExp(1).getString().equals("false-if-unknown")) 
      rval.setTriValue(rval.getTriValue().falseIfUnknown());
      
    if (exp.getSExp(1).getString().equals("true-if-unknown")) 
      rval.setTriValue(rval.getTriValue().trueIfUnknown());

    return rval;
  }
  

  // if true, return all statements that are true
  // if false, return all statements that are false
  private ReturnedValues matchRule(SExp exp)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();
    StatementList stmtlist = addedStatementList;
    StatementList true_stmt_list = new StatementList();
    StatementList false_stmt_list = new StatementList();
    StatementList restrict_stmt_list = new StatementList();
    SExp pattern = new SExp();
    
    if ( ! ( exp.expect(exp.SEXP_SEXP) &&
	     pattern.parse(exp.current().toString())))
      throw new ProfilesException("expect a valid pattern" + 
				  exp.current().toString());

    for (int i=1; i <= stmtlist.size(); i++) {
      try { 
	if (stmtlist.getStatement(i).restrictMatch(pattern))
	  true_stmt_list.addStatement(stmtlist.getStatement(i));
	else
	  false_stmt_list.addStatement(stmtlist.getStatement(i));
      } catch (SExpException e) {
	if (e.getMessage().equals("has !"))
	  restrict_stmt_list.addStatement(stmtlist.getStatement(i));
	if (e.getMessage().equals("no !")) 
	  false_stmt_list.addStatement(stmtlist.getStatement(i));
      }
    }
    
    if (restrict_stmt_list.size() > 0) {
      rval.setTriValue(new TriValue (false));
      rval.setStatementList(restrict_stmt_list);
      return rval;
    }
    if (true_stmt_list.size() > 0) {
      rval.setTriValue(new TriValue (true));
      rval.setStatementList(true_stmt_list);
      return rval;
    }
    rval.setTriValue(new TriValue (false));
    rval.setStatementList(false_stmt_list);
    return rval;
  }


  // (and rule1 rule2 rule3)
  private ReturnedValues combineRuleAnd(SExp exp)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();
    rval.setTriValue(new TriValue (true));
    for (int i=2; i<=exp.size(); i++) {
      ReturnedValues nrval = evalRule(exp.getSExp(i));
      if (nrval.getTriValue().equals(rval.getTriValue())) {
	rval.getStatementList().merge(nrval.getStatementList());
	continue;
      }
      rval.setTriValue(rval.getTriValue().and(nrval.getTriValue()));
      if (nrval.getTriValue().equals(rval.getTriValue()))
	rval.setStatementList(nrval.getStatementList());
    }
    return rval;
  }
  
  // (or rule1 rule2 rule3)
  private ReturnedValues combineRuleOr(SExp exp)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();    
    rval.setTriValue(new TriValue (false));
    for (int i=2; i<=exp.size(); i++) {
      ReturnedValues nrval = evalRule(exp.getSExp(i));
      if (nrval.getTriValue().equals(rval.getTriValue())) {
	rval.getStatementList().merge(nrval.getStatementList());
	continue;
      }
      rval.setTriValue(rval.getTriValue().or(nrval.getTriValue()));
      if (nrval.getTriValue().equals(rval.getTriValue()))
	rval.setStatementList(nrval.getStatementList());
    }
    return rval;
  }
  

  // now just return a default PICS label
  // example:
  // (invoke "load-label" STATEMENT-LIST URL "http://www.musac.org/"
  //         (EMBEDDED))
  // right now STATEMENT-LIST and URL variables are not used.
  private ReturnedValues invokeRule(SExp exp)
    throws ProfilesException
  {
    
    ReturnedValues rval = new ReturnedValues();
    rval.setTriValue(new TriValue (false));

    // (invoke "check-hash" STATEMENT-LIST) 
    // prepend "check-hash" for all that has md5 field in it
    if (exp.getSExp(2).getString().equals("check-hash")) {
      StatementList stmt_list = addedStatementList;  //need to clone it
      for (int i=1; i<=stmt_list.size(); i++) {
	SExp pattern = new SExp();
	pattern.parse("(*((\"load-label\" *)((version \"PICS-1.1\") * (md5 *)*)*)))");
	try {
	  if (stmt_list.getStatement(i).restrictMatch(pattern)) {
	    Statement new_stmt = stmt_list.getStatement(i);  //need to clone it
	    new_stmt.prependContext("check-hash");
	    rval.getStatementList().addStatement(new_stmt);
	    rval.setTriValue(new TriValue (true));
	  }
	} catch (SExpException e) {}
      }
      return rval;
    }

    if (exp.getSExp(2).getString().equals("endorse-label")) {
      StatementList stmt_list = addedStatementList;  //need to clone it
      SExp endorser = exp.getSExp(4);
      for (int i=1; i<=stmt_list.size(); i++) {
	for (int j=0; j<endorsed_rater.length; j++) {
	  SExp pattern = new SExp();
	  pattern.parse("(* ((\"load-label\" *) ((version \"PICS-1.1\") * (by \"" +
			endorsed_rater[j] + "\" ) * ) ) )");
	  try {
	    if (stmt_list.getStatement(i).restrictMatch(pattern)) {
	      Statement new_stmt = stmt_list.getStatement(i);  //need to clone it
	      new_stmt.prependContext("endorse-label");
	      new_stmt.getContent().setSExp(1, endorser);
	      rval.getStatementList().addStatement(new_stmt);
	      rval.setTriValue(new TriValue (true));
	    }
	  } catch (SExpException e) {}
	}
      }
      return rval;
    }


    SExp content = new SExp();
    String invokePolicy = "";
    
    try {
      exp.nextToken();
      invokePolicy = PROGRAM_DATABASE.getProgram(exp.current().getString());
    } catch (Exception e) {
      throw new ProfilesException(e.getMessage());
    }
    

    if (! content.parse(invokePolicy))
      throw new ProfilesException ("invalid s-expression label file");
   
    
    for (int i=1; i<=content.size(); i++) {
      LabelParser lp = new LabelParser();
      PICSLabelList ll = new PICSLabelList();
      try { ll.parse(content.getSExp(i)); }
      catch (PICSParseException e) {
	throw new ProfilesException(e.getMessage()); }
      lp.addLabel(ll);
      SExp res = new SExp();
      try { res = lp.search(ARG_URL, "...", true); }
      catch (PICSLabelException e) {
	throw new ProfilesException(e.getMessage());
      }
      SExp stmt_header = new SExp(SExp.SEXP_SEXP);
      stmt_header.addSExp(new SExp(SExp.SEXP_STRING, "load-label"));
      stmt_header.addSExp(new SExp(SExp.SEXP_STRING, ARG_URL.toString()));
      stmt_header.addSExp(exp.getSExp(6).getSExp(1));
      SExp stmt_content = new SExp(SExp.SEXP_SEXP);
      stmt_content.addSExp(stmt_header);
      stmt_content.addSExp(res.getSExp(1));
      Statement stmt = new Statement("load-label", stmt_content);
      rval.getStatementList().addStatement(stmt);
      rval.setTriValue(new TriValue (true));
    }
    return rval;
    }
  
  // ( url-match  ) => return true
  // (url-match URL [url]+)
  private ReturnedValues explicitRule(SExp exp)
       throws ProfilesException
  {
    ReturnedValues rvals = new ReturnedValues();

    // if action follows no argument, return a TriValue and an empty stmtList
    if (exp.size() == 1) {
      rvals.setTriValue(new TriValue('t'));
      return rvals;
    }

    // handle the more complicated case, url-match with URLs
    boolean matched = false;
    URL matchingURL;
    SExp matchList = null;
    
    if ((exp.getSExp(2).ttype != exp.SEXP_IDENT) ||
	(! exp.getSExp(2).getString().equals("URL")))
      throw new ProfilesException("expect keyward URL");
    
    if (exp.getSExp(3).ttype != exp.SEXP_SEXP)
      throw new ProfilesException("expect a list of matching URLs");
    matchList = exp.getSExp(3);

    for (int index=1; index <= matchList.size(); index++) {
      if (matchList.getSExp(index).ttype != SExp.SEXP_STRING)
	throw new ProfilesException(matchList.getSExp(index).toString() +
				    " is not an URL");      
      try {
	matchingURL = new URL(matchList.getSExp(index).getString());
      } catch (MalformedURLException e) {
	throw new ProfilesException("Unable to parse the URL" + 
				    matchList.getSExp(index).toString());
      }
      
      if (ARG_URL.toString().startsWith(matchingURL.toString())) {
	Statement newStmt = new Statement();
	newStmt.prependContext(PROGRAM_NAME);
	newStmt.getContent().addSExp(new SExp(newStmt.SEXP_STRING,"url-match"));
	newStmt.getContent().addSExp(new SExp(newStmt.SEXP_STRING, 
				     matchingURL.toString()));
	rvals.getStatementList().addStatement(newStmt);
      }
    }
    if (rvals.getStatementList().size() == 0)
      rvals.setTriValue(new TriValue('f'));
    else
      rvals.setTriValue(new TriValue('t'));
    
    return rvals;
  }

  /* was allow/block
  private ReturnedValues explicitRule(SExp exp)
       throws ProfilesException
  {
    ReturnedValues rvals = new ReturnedValues();
    boolean isActionAllow = false;

    // isActionAllow = true if first argument is allow, otherwise false
    if (exp.current().getString().equals("allow"))
      isActionAllow = true;
    
    // if action follows no argument, return a TriValue and an empty stmtList
    if (exp.size() == 1) {
      rvals.setTriValue(new TriValue(isActionAllow));
      return rvals;
    }

    // handle the more complicated case, allow/block with URLs
    boolean matched = false;
    URL matchingURL;
    
    if ((exp.getSExp(2).ttype != exp.SEXP_IDENT) ||
	(! exp.getSExp(2).getString().equals("URL")))
      throw new ProfilesException("expect keyward URL");
    
    for (int index=3; index <= exp.size(); index++) {
      if (exp.getSExp(index).ttype != exp.SEXP_STRING)
	throw new ProfilesException("expect a URL");
      
      try {
	matchingURL = new URL(exp.getSExp(index).getString());
      } catch (MalformedURLException e) {
	throw new ProfilesException("Unable to parse URL");
      }
      
      if (ARG_URL.toString().startsWith(matchingURL.toString())) {
	Statement newStmt = new Statement();
	newStmt.prependContext(PROGRAM_NAME);
	if (isActionAllow)
	  newStmt.getContent().addSExp(new SExp(newStmt.SEXP_STRING, "allow"));
	else
	  newStmt.getContent().addSExp(new SExp(newStmt.SEXP_STRING, "block"));
	newStmt.getContent().addSExp(new SExp(newStmt.SEXP_STRING, 
				     matchingURL.toString()));
	rvals.getStatementList().addStatement(newStmt);
      }
    }
    
    if (rvals.getStatementList().size() == 0)
      rvals.setTriValue(new TriValue(! isActionAllow));
    else
      rvals.setTriValue(new TriValue(isActionAllow));
    
    return rvals;
  }
  */

  public static void halt()
  {
      System.out.print("Press return to continue > ");
      System.out.flush();
      try { int j = System.in.read(); }
      catch (Exception e) { }
      System.out.print("\n\n");
  }

  public static String readln(String msg, String def)
  {
    String s = "";
    System.out.print(msg + " [" + def + "] : ");
    System.out.flush();
    try { s = new DataInputStream(System.in).readLine(); }
    catch (Exception e) { }
    if (s.equals("")) s = def;
    return s;
  }    
  
  public static void main(String[] argv)
       throws Exception
  {
    // extract arguments from invocation

    String progFileName;
    String labelFileName;
    String targetURLString;

    // 1st argument: Profile filename
    if (argv.length < 1) 
      progFileName = readln("Enter the filename for the Profile", "prog5-1");
    else 
      progFileName = argv[0];
    
    // 2nd argument: name of the file containinig a list of PICS labels
    if (argv.length < 2)
      labelFileName = readln("Enter the filename for the PICS labels", 
			     "label5-1");
    else
      labelFileName = argv[1];
    
    // 3rd argument: target URL of interest
    if (argv.length < 3)
      targetURLString = readln("Enter the target URL", 
			       "http://www.w3.org/Overview.html");
    else
      targetURLString = argv[2];

    // endorser are now set to "Alice" and "Bob" by default
    System.out.println("Endorsers for this demo are set by default to be \"Alice\" and \"Bob\"");
    halt();
    String endorser[] = {"Alice", "Bob"};
    
    // local variable
    URL requestURL = new URL(targetURLString);
    String prog;
    StatementList initStmtList = new StatementList();
    ProgramDatabase initProgDBase = new ProgramDatabase();
    Profiles myProfiles = new Profiles(initProgDBase, initStmtList, 
				       requestURL);
    myProfiles.DEBUG = true;
    myProfiles.DEBUG_TERMINAL = true;
    myProfiles.endorsed_rater = endorser;

    SExp myProgram = new SExp();
    try { 
      FileInputStream insPolicy = new FileInputStream(progFileName);
      if (! myProgram.parseMultiple(insPolicy))   // BUG! BUG!
	throw new ProfilesException ("invalid s-expression program file");    
      initProgDBase.insertProgram("Profiles", myProgram.toString());      
    } 
    catch (IOException e) { 
      throw new ProfilesException("Unable to open file: "+progFileName+"\n"); 
    }
    
    try { 
      FileInputStream insLabel = new FileInputStream(labelFileName);
      SExp labelSExp = new SExp();
      if (! labelSExp.parseMultiple(insLabel))   // BUG! BUG!
	throw new ProfilesException ("invalid s-expression label file");    
      initProgDBase.insertProgram("load-label", labelSExp.toString());     
    } 
    catch (IOException e) { 
      throw new ProfilesException("Unable to open file: "+labelFileName+"\n"); 
    }

    System.out.println("The Program you are running: ");
    System.out.println(initProgDBase.getProgram("Profiles"));
    halt();

    System.out.println("The Labels you are matching against: ");
    System.out.println(initProgDBase.getProgram("load-label"));
    halt();

    
    String result = myProfiles.eval("Profiles").toString();
    System.out.println("The answer is: \n");
    System.out.println(result);
    // myProfiles.halt();
    // System.out.print("\nThe new statement genereated:");
    // System.out.println(myProfiles.addedStatementList.toString());
  }
} // class Profiles

  





