// Profiles.java
// by Yang-hua Chu
// last update: 10/11/96

package w3c.pics.referee;

import java.io.* ;
import java.net.*;
import java.util.*;

public class Profiles {
  
  public boolean DEBUG=false;
  public URL ARG_URL = null;
  public StatementList INITIAL_STATEMENT_LIST = new StatementList();
  private ProgramDatabase PROGRAM_DATABASE = new ProgramDatabase();
  public String PROGRAM_NAME = "Profiles";
  public StatementList addedStatementList = new StatementList();
  private SExp traceSExp = new SExp(SExp.SEXP_SEXP);

  public String trace()
  {
    String s = "";
    for (int i=1; i<traceSExp.size(); i=i+2) {
      s += "eval: " + traceSExp.getSExp(i).toString() + "\n\n";
      s += "result: " + traceSExp.getSExp(i+1).toString() + "\n";
      s += "---------------------------------------------\n\n";
    }
    return s;
  }

  public Profiles(ProgramDatabase inputProgDatabase, 
		  StatementList statementList, 
		  URL inputURL) 
    throws ProfilesException
  {
    PROGRAM_DATABASE = inputProgDatabase;
    INITIAL_STATEMENT_LIST = statementList;
    ARG_URL = inputURL;
  }
  

  public ReturnedValues eval(String action)
    throws SExpException, ProfilesException
  {
    SExp myProgram = new SExp();
    ReturnedValues rvalue = new ReturnedValues();
    String policyString = "";
    
    try {
      policyString = PROGRAM_DATABASE.getProgram(action);
    } catch (ProfilesException e) {
      throw new ProfilesException("Unable to find the program " +action +
				  " in the program database"); 
    }
    
    if (! myProgram.parse(policyString))
	throw new ProfilesException ("invalid s-expression program file");
    
    for (int i=1; i <= myProgram.size(); i++) {
      if (DEBUG) { 
	System.out.println("Eval:" + myProgram.getSExp(i).toString() + "\n");
	halt();
      }
      rvalue = evalRule(myProgram.getSExp(i));
      addedStatementList.merge(rvalue.getStatementList());
      if (DEBUG) { 
	System.out.println("Result:" + rvalue.toString() + "\n");
	halt();
      }
      traceSExp.addSExp(myProgram.getSExp(i));
      traceSExp.addSExp(new SExp(rvalue.toString()));
    }
    return rvalue;
  }
  

  // rule::let-rule | explicit-rule | combine-rule | invoke-rule | 
  //       install-rule | projection-rule | match-rule | variable-name 
  public ReturnedValues evalRule(SExp exp) 
    throws ProfilesException
  {

    if (exp.expect(exp.SEXP_IDENT, "allow") ||
	exp.expect(exp.SEXP_IDENT, "block"))
      return explicitRule(exp);
    
    /*    if (exp.expect(exp.SEXP_IDENT, "let"))
	  return letRule(exp);
	  */

    if (exp.expect(exp.SEXP_IDENT, "or"))
      return combineRuleOr(exp);
    
    if (exp.expect(exp.SEXP_IDENT, "and"))
      return combineRuleAnd(exp);
    
    if (exp.expect(exp.SEXP_IDENT, "invoke"))
      return invokeRule(exp);
    
    /*
      if ((exp.expect(exp.SEXP_IDENT, "install-program")) ||
	(exp.expect(exp.SEXP_IDENT, "install-interpreter")))
      return installRule(exp, exp.getString());

    if ((exp.expect(exp.SEXP_IDENT, "tri-value")) ||
	(exp.expect(exp.SEXP_IDENT, "statement-list")))
      return projectionRule(exp, exp.getString());
      */
    
    if (exp.expect(exp.SEXP_IDENT, "match"))
      return matchRule(exp);
    
    throw new ProfilesException("unknown rule" + exp.toStringShort());
  }


  // if true, return all statements that are true
  // if false, return all statements that are false
  private ReturnedValues matchRule(SExp exp)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();
    StatementList stmtlist = addedStatementList;
    StatementList true_stmt_list = new StatementList();
    StatementList false_stmt_list = new StatementList();
    StatementList restrict_stmt_list = new StatementList();
    SExp pattern = new SExp();
    
    if ( ! ( exp.expect(exp.SEXP_SEXP) &&
	     pattern.parse(exp.current().toString())))
      throw new ProfilesException("expect a valid pattern" + 
				  exp.current().toString());

    for (int i=1; i <= stmtlist.size(); i++) {
      try { 
	if (stmtlist.getStatement(i).restrictMatch(pattern))
	  true_stmt_list.addStatement(stmtlist.getStatement(i));
	else
	  false_stmt_list.addStatement(stmtlist.getStatement(i));
      } catch (SExpException e) {
	if (e.getMessage().equals("has !"))
	  restrict_stmt_list.addStatement(stmtlist.getStatement(i));
	if (e.getMessage().equals("no !")) 
	  false_stmt_list.addStatement(stmtlist.getStatement(i));
      }
    }
    
    if (restrict_stmt_list.size() > 0) {
      rval.setTriValue(new TriValue (false));
      rval.setStatementList(restrict_stmt_list);
      return rval;
    }
    if (true_stmt_list.size() > 0) {
      rval.setTriValue(new TriValue (true));
      rval.setStatementList(true_stmt_list);
      return rval;
    }
    rval.setTriValue(new TriValue (false));
    rval.setStatementList(false_stmt_list);
    return rval;
  }


  // (and rule1 rule2 rule3)
  private ReturnedValues combineRuleAnd(SExp exp)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();
    rval.setTriValue(new TriValue (true));
    for (int i=2; i<=exp.size(); i++) {
      ReturnedValues nrval = evalRule(exp.getSExp(i));
      if (nrval.getTriValue().equals(rval.getTriValue())) {
	rval.getStatementList().merge(nrval.getStatementList());
	continue;
      }
      rval.setTriValue(rval.getTriValue().and(nrval.getTriValue()));
      if (nrval.getTriValue().equals(rval.getTriValue()))
	rval.setStatementList(nrval.getStatementList());
    }
    return rval;
  }
  
  // (or rule1 rule2 rule3)
  private ReturnedValues combineRuleOr(SExp exp)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();    
    rval.setTriValue(new TriValue (false));
    for (int i=2; i<=exp.size(); i++) {
      ReturnedValues nrval = evalRule(exp.getSExp(i));
      if (nrval.getTriValue().equals(rval.getTriValue())) {
	rval.getStatementList().merge(nrval.getStatementList());
	continue;
      }
      rval.setTriValue(rval.getTriValue().or(nrval.getTriValue()));
      if (nrval.getTriValue().equals(rval.getTriValue()))
	rval.setStatementList(nrval.getStatementList());
    }
    return rval;
  }
  

  // now just return a default PICS label
  private ReturnedValues invokeRule(SExp exp)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();
    rval.setTriValue(new TriValue (true));
    SExp content = new SExp();
    String invokePolicy = "";
    
    try {
      exp.nextToken();
      invokePolicy = PROGRAM_DATABASE.getProgram(exp.current().getString());
    } catch (Exception e) {
      throw new ProfilesException(e.getMessage());
    }
    
    if (! content.parse(invokePolicy))
      throw new ProfilesException ("invalid s-expression label file");
    
    for (int i=1; i<=content.size(); i++) {
      Statement stmt = new Statement("load-label", content.getSExp(i));
      rval.getStatementList().addStatement(stmt);
    }
    return rval;
  }
  
  // ( allow/block )
  // (allow/block  URL [url]+)
  private ReturnedValues explicitRule(SExp exp)
       throws ProfilesException
  {
    ReturnedValues rvals = new ReturnedValues();
    boolean isActionAllow = false;

    // isActionAllow = true if first argument is allow, otherwise false
    if (exp.current().getString().equals("allow"))
      isActionAllow = true;
    
    // if action follows no argument, return a TriValue and an empty stmtList
    if (exp.size() == 1) {
      rvals.setTriValue(new TriValue(isActionAllow));
      return rvals;
    }

    // handle the more complicated case, allow/block with URLs
    boolean matched = false;
    URL matchingURL;
    
    if ((exp.getSExp(2).ttype != exp.SEXP_IDENT) ||
	(! exp.getSExp(2).getString().equals("URL")))
      throw new ProfilesException("expect keyward URL");
    
    for (int index=3; index <= exp.size(); index++) {
      if (exp.getSExp(index).ttype != exp.SEXP_STRING)
	throw new ProfilesException("expect a URL");
      
      try {
	matchingURL = new URL(exp.getSExp(index).getString());
      } catch (MalformedURLException e) {
	throw new ProfilesException("Unable to parse URL");
      }
      
      if (ARG_URL.toString().startsWith(matchingURL.toString())) {
	Statement newStmt = new Statement();
	newStmt.appendContext(PROGRAM_NAME);
	if (isActionAllow)
	  newStmt.getContent().addSExp(new SExp(newStmt.SEXP_STRING, "allow"));
	else
	  newStmt.getContent().addSExp(new SExp(newStmt.SEXP_STRING, "block"));
	newStmt.getContent().addSExp(new SExp(newStmt.SEXP_STRING, 
				     matchingURL.toString()));
	rvals.getStatementList().addStatement(newStmt);
      }
    }
    
    if (rvals.getStatementList().size() == 0)
      rvals.setTriValue(new TriValue(! isActionAllow));
    else
      rvals.setTriValue(new TriValue(isActionAllow));
    
    return rvals;
  }

  public void halt()
  {
    if (DEBUG) {
      System.out.print("Press return to continue > ");
      System.out.flush();
      try { int j = System.in.read(); }
      catch (Exception e) { }
    }
  }
  
  public static void main(String[] argv)
       throws Exception
  {
    boolean DEBUG= true;
    // local variable
    String prog;
    StatementList initStmtList = new StatementList();
    URL requestURL = new URL("http://playboy/fun.html");

    ProgramDatabase initProgDBase = new ProgramDatabase();
    Profiles myProfiles = new Profiles(initProgDBase, initStmtList, 
				       requestURL);
    myProfiles.DEBUG = true;

    if (argv == null) {
      System.err.println("Please add a valid filename as the first argument");
      return;
    }

    SExp myProgram = new SExp();
    try { 
      FileInputStream insPolicy = new FileInputStream(argv[0]);
      if (! myProgram.parseMultiple(insPolicy))   // BUG! BUG!
	throw new ProfilesException ("invalid s-expression program file");    
      initProgDBase.insertProgram("Profiles", myProgram.toString());      
    } 
    catch (IOException e) { 
      throw new ProfilesException("Unable to open file: "+argv[0]+"\n"); 
    }
    
    try { 
      FileInputStream insLabel = new FileInputStream(argv[1]);
      SExp labelSExp = new SExp();
      if (! labelSExp.parseMultiple(insLabel))   // BUG! BUG!
	throw new ProfilesException ("invalid s-expression label file");    
      initProgDBase.insertProgram("load-label", labelSExp.toString());     
    } 
    catch (IOException e) { 
      throw new ProfilesException("Unable to open file: label-file\n"); 
    }

    System.out.print("The Program you are running: ");
    System.out.println(initProgDBase.getProgram("Profiles"));
    myProfiles.halt();

    System.out.print("The Labels you are matching against: ");
    System.out.println(initProgDBase.getProgram("load-label"));
    myProfiles.halt();

    
    String result = myProfiles.eval("Profiles").toString();
    System.out.print("\nThe answer is: ");
    System.out.println(result);
    // myProfiles.halt();
    // System.out.print("\nThe new statement genereated:");
    // System.out.println(myProfiles.addedStatementList.toString());
  }
} // class Profiles

  





