// ProgramDatabase.java
// by Yang-hua Chu
// created: 11/2/96
// last update: 11/2/96

package w3c.pics.referee;

import java.io.* ;
import java.net.*;
import java.util.*;
import w3c.pics.util.*;

public class ProgramDatabase {

  String names[] = {};
  String programs[] = {};  
  String interpreters[] = {};
  
  public String toString()
  {
    String s = "";
    for (int i=0; i<size(); i++) {
      s += "Action     : " + names[i] + "\n";
      s += "Policy     : " + programs[i] + "\n";
      s += "Interpreter: " + interpreters[i] + "\n";
    }
    return s;
  }

  public int size()
  {
    return names.length;
  }

  public int insertProgram(String name, String program)
  {
    int index = lookup(name);
  
    if (index != -1) {   // program name already exists, over-ridiing
      programs[index] = program;
      return index;
    }

    return insert(name, program, null);
  }

  public int insertInterpreter(String name, String interpreter)
  {
    int index = lookup(name);
    
    if (index != -1) {   // program name already exists, over-ridiing
      interpreters[index] = interpreter;
      return index;
    }
    return insert(name, null, interpreter);
  }

  public int insert(String name, String program, String interpreter)
  {
    int index = lookup(name);
    
    if (index != -1) {   // program name already exists, over-ridiing
      programs[index] = program;
      interpreters[index] = interpreter;
      return index;
    }
    String nnames[] = new String[size()+1];
    String nprograms[] = new String[size()+1];
    String ninterpreters[] = new String[size()+1];

    System.arraycopy(names, 0, nnames, 0, size());
    System.arraycopy(programs, 0, nprograms, 0, size());
    System.arraycopy(interpreters, 0, ninterpreters, 0, size());

    nnames[size()] = name;
    nprograms[size()] = program;
    ninterpreters[size()] = interpreter;

    names = nnames;
    programs = nprograms;
    interpreters = ninterpreters;
    return size()-1;
  }
    
  
  /*
    public ProgramDatabase clone()
    {
    return this.clone();
    }
  */

  public String getProgram(String name)
    throws ProfilesException
  {
    int index = lookup(name);
    if (index == -1)
      throw new ProfilesException("ProgramDatabase error: program " + name +
				  " not found.");
    return programs[index];
  }

  private int lookup(String name)
  {
    for (int i=0; i < size(); i++)
      if (names[i].equals(name))
	return i;
    return -1;
  }

}
