package w3c.pics.referee;

import java.util.*;
import java.net.*;

import w3c.www.http.*;
import w3c.jigsaw.http.*;

import w3c.jigsaw.resources.*;
import w3c.jigsaw.forms.*;
import w3c.jigsaw.html.*;
 
import w3c.pics.util.*;


public class RefereeResource extends PostableResource {

   // The message attribute index
   protected static int ATTR_MESSAGE = -1 ;
   // The headers attribute index
   protected static int ATTR_HEADERS = -1 ;
   
   static {
     Attribute a   = null ;
     Class     cls = null ;
     
     try {
       cls = Class.forName("w3c.pics.referee.RefereeResource");
     } catch (Exception ex) {
       ex.printStackTrace() ;
       System.exit(1) ;
     }
     // Register the message attribute:
     a = new StringAttribute("message", "Hello", Attribute.EDITABLE);
     ATTR_MESSAGE = AttributeRegistry.registerAttribute(cls, a) ;
                 // Register the headers attribute:
     a = new StringArrayAttribute("headers", null, Attribute.EDITABLE) ;
     ATTR_HEADERS = AttributeRegistry.registerAttribute(cls, a) ;
   }
   
   
   /**
    * Display the resource message with the request headers.
    * @param request The request to handle.
    * @exception HTTPException If processing the request failed.
    */
   
   public Reply get(Request request) 
     throws HTTPException
   {
     // Create the HTML generator, and set titles:
     HtmlGenerator g = new HtmlGenerator("FancyResource");
     g.append("<h1>FancyResource output12</h1>");
     // Emit the message:
     g.append("<p>"+getValue(ATTR_MESSAGE, null));
     // Now emit the requested headers:
     String headers[] = (String[]) getValue(ATTR_HEADERS, null) ;
     if ( headers != null ) {
       g.append("<hr><h2>Request headers:</h2><dl>");
       for (int i = 0 ; i < headers.length ; i++) {
	 g.append("<dt>"+headers[i]+"</dt><dd>");
	 String value = request.getValue(headers[i]);
	 g.append((value == null) 
		  ? "<em>undefined</em>"
		  : value) ;
       }
     }
     Reply reply = createDefaultReply(request, HTTP.OK) ;
     reply.setStream(g) ;
     return reply ;
   }
 
  
   public synchronized Reply handle (Request request, URLDecoder data) 
     throws HTTPException
   {
     // input
     String URL = "";
     String action = "";
     String policy = "";
     String labels = "";
     
     // output
     String answer = "";
     String trace = "";
     String raters = "";

     Enumeration e = data.keys() ;
     while ( e.hasMoreElements() ) {
       String name  = (String) e.nextElement() ;
       System.out.println("Name = " + name + data.getValue(name));
       if (name.equals("URL"))
	 URL = data.getValue(name) ;
       if (name.equals("action"))
	 action = data.getValue(name) ;
       if (name.equals("policy"))
	 policy = data.getValue(name) ;
       if (name.equals("labels"))
	 labels = data.getValue(name) ;
       if (name.equals("raters"))
	 raters = data.getValue(name) ;
     }
     
     URL requestURL = null;
     try { requestURL = new URL(URL); }
     catch (Exception e2) {
       System.err.println("invalud URL: " + URL);
     }
     
     
     // local variable
     StatementList initStmtList = new StatementList();
     ProgramDatabase initProgDBase = new ProgramDatabase();

     try {
       SExp tmp = new SExp();
       tmp.parseMultiple(policy);
       policy = tmp.toString();
       SExp tmp2 = new SExp();
       tmp2.parseMultiple(labels);
       labels = tmp2.toString();
     } catch (Exception e4) {
       System.err.println("bad policy or bad label");
     }
     
     initProgDBase.insertProgram("Profiles", policy);
     initProgDBase.insertProgram("load-label", labels);

     System.out.println("The policy is: \n" + policy);
     System.out.println("The label is: \n" + labels);
     
     try {
       Profiles myProfiles = new Profiles(initProgDBase, initStmtList, 
					  requestURL);
       // myProfiles.DEBUG=true;
       
       SExp rater_sexp = new SExp();
       rater_sexp.parse(raters);
       String tmp_string_arr[] = new String[rater_sexp.size()];
       for (int i=1; i<=rater_sexp.size(); i++)
	 tmp_string_arr[i-1] = rater_sexp.getSExp(i).getString();
       myProfiles.endorsed_rater = tmp_string_arr;
       
       
       answer = myProfiles.eval("Profiles").toString();
       trace = myProfiles.trace();
     } catch (Exception e3) { 
       trace = e3.getMessage() + " error msg";   // need more work
     }
     
     Reply         r = request.makeReply(HTTP.OK);
     HtmlGenerator g = new HtmlGenerator ("REFEREE evaluation") ;
     g.append ("<p>The answer is:<p><pre>");
     g.appendAndEscape(answer);
     g.append(" </pre><hr>" + "A trace is as followed:</p><pre>");
     g.appendAndEscape(trace);
     g.append("</pre>");
     r.setStream (g) ;
     return r ;
   }

   
   // public Reply createDefaultReply(Request request, int status) {
   // Reply reply = super.createDefaultReply(request, status);
   //   reply.setNoStore();
     // Be firndly to HTTP/1.0 caches too:
   //  reply.addPragma("no-cache");
   //  }

   
 }
