// TriValue.java
// by Yang-hua Chu
// last update: 10/30/96

package w3c.pics.referee;

import java.io.* ;
import java.util.*;

public class TriValue {
  final static int TRIVALUE_TRUE = 2;
  final static int TRIVALUE_FALSE = 0;
  final static int TRIVALUE_UNKNOWN = 1;

  private int val = TRIVALUE_TRUE;
  
  public TriValue()
  {
    val = TRIVALUE_FALSE;
  }

  public TriValue(char c)
  {
    switch (c) {
    case 'T': case 't': val = TRIVALUE_TRUE; break;
    case 'F': case 'f': val = TRIVALUE_FALSE; break;
    case 'U': case 'u': val = TRIVALUE_UNKNOWN; break;
    }
  }

  public TriValue(boolean bool)
  {
    if (bool)
      val = TRIVALUE_TRUE;
    else
      val = TRIVALUE_FALSE;
  }
  
  public Object clone()
  {
    switch (val) {
    case TRIVALUE_TRUE: return new TriValue('t');
    case TRIVALUE_FALSE: return new TriValue('f');
    default: TRIVALUE_UNKNOWN: return new TriValue('u');
    }
  }

  public TriValue and(TriValue tv) 
  {
    if (tv.val < this.val)
      return (TriValue) tv.clone();
    else
      return (TriValue) clone();
  }

  public TriValue or(TriValue tv) 
  {
    if (tv.val > this.val)
      return (TriValue) tv.clone();
    else
      return (TriValue) clone();
  }

  public TriValue not()
  {
    switch (val) {
    case TRIVALUE_UNKNOWN: return new TriValue('u');
    case TRIVALUE_TRUE:    return new TriValue('f');
    case TRIVALUE_FALSE:   return new TriValue('t');
    default:  // should not happen
      return new TriValue('f');
    }
  }

  public TriValue falseIfUnknown() 
  {
    if (val == TRIVALUE_UNKNOWN)
      return new TriValue('f');
    else
      return (TriValue) clone();
  }

  public TriValue trueIfUnknown() 
  {
    if (val == TRIVALUE_UNKNOWN)
      return new TriValue('t');
    else
      return (TriValue) clone();
  }

  public boolean equals(TriValue tv) 
  {
    return (tv.val == this.val);
  }
  
  public String toString()
  {
    switch (val) {
    case TRIVALUE_TRUE: return "\"TRUE\"";
    case TRIVALUE_UNKNOWN: return "\"UNKNOWN\"";
    default: return "\"FALSE\"";
    }
  }
  
}
